@@ -12,8 +12,12 @@ import json
 import os
 import re
 import shlex
+import shutil
+import subprocess
 from glob import glob
 from os.path import join
+from pathlib import Path
+from typing import Iterable
 
 import click
 import requests
@@ -33,6 +37,7 @@ from .utils import (
     inside_ap_datapkg,
     logging_subprocess_run,
     production_name_type,
+    production_to_versions,
     validate_environment,
 )
 
@@ -92,11 +97,194 @@ def login():
     click.secho(f"Logged in successfully as {r.json()['username']}", fg="green")
 
 
+@main.command()
+@click.argument("working_group", type=str, default=None, nargs=1, required=False)
+@click.argument(
+    "production_name", type=production_name_type, default=None, nargs=1, required=False
+)
+def versions(working_group: str, production_name: production_name_type) -> None:
+    """List the available tags of the Analysis Productions repository by running 'lb-ap versions' \n
+    List the available tags for a specific production by running 'lb-ap versions <WORKING_GROUP> <YOUR_PRODUCTION>'"""
+    # TODO the comments below are to facilitate mapping each version to a production but requires some changes
+    # in the api website at a later date
+    if not production_name:
+        # from apd import AnalysisData
+        # from apd.ap_info import fetch_wg_info
+        # versions_to_productions = {}
+        if not working_group:
+            raise click.ClickException(
+                "version querying currently only works with working group,"
+                " and production name specified! lb-ap versions WORKING_GROUP PRODUCTION_NAME"
+            )
+        #    from LbAPCommon.config import known_working_groups
+        #    for wg in known_working_groups:
+        #        productions = fetch_wg_info(wg)
+        #        for production in productions:
+        #            dataset = AnalysisData(wg, production)
+        #            for version in dataset.available_tags["versions"]:
+        #                versions_to_productions[version] = [wg, production]
+        else:
+            raise click.ClickException(
+                "version querying currently only works with working group,"
+                " and production name specified! lb-ap versions WORKING_GROUP PRODUCTION_NAME"
+            )
+
+            # productions = fetch_wg_info(wg)
+            # for production in productions:
+            #    dataset = AnalysisData(wg, production)
+            #    for version in dataset.available_tags["versions"]:
+            #        versions_to_productions[version] = [wg, production]
+        # click.echo(f"The available versions are:")
+        # for version, wg_production in versions_to_productions.items():
+        #    click.echo(f"{version}: {wg_production[0]} {wg_production[1]}")
+    else:
+        versions = production_to_versions(working_group, production_name)
+        click.echo(f"The available versions for {production_name} are: ")
+        for version in versions:
+            click.echo(f"    {version}")
+
+
+@main.command()
+@click.argument("version", type=str, nargs=1)
+@click.argument("working_group", type=str, nargs=1)
+@click.argument("production_name", type=production_name_type, nargs=1)
+@click.argument("branch_name", type=str, nargs=1)
+@click.argument(
+    "clone_type", type=click.Choice(["ssh", "https", "krb5"]), default="ssh", nargs=1
+)
+def clone(
+    version: str,
+    working_group: str,
+    production_name: production_name_type,
+    branch_name: str,
+    clone_type: str,
+) -> None:
+    """Clone the AnalysisProductions repository and do lb-ap checkout
+    with the specified version, production and branch name.
+
+    Specify clone_type as either ssh, https or krb5."""
+    if clone_type not in ["ssh", "https", "krb5"]:
+        raise click.ClickException(f"{clone_type} is not a valid cloning option!")
+    if version not in production_to_versions(working_group, production_name):
+        raise click.ClickException(
+            f"Version {version} does not correspond to the {production_name} production!."
+        )
+    click.secho(
+        f"Cloning AnalysisProductions, changing {production_name} to that,"
+        f" of {version} and checking out the {branch_name} branch.",
+        fg="yellow",
+    )
+    if clone_type == "ssh":
+        clone_url = "ssh://git@gitlab.cern.ch:7999/lhcb-datapkg/AnalysisProductions.git"
+    elif clone_type == "https":
+        clone_url = "https://gitlab.cern.ch/lhcb-datapkg/AnalysisProductions.git"
+    elif clone_type == "krb5":
+        clone_url = "https://:@gitlab.cern.ch:8443/lhcb-datapkg/AnalysisProductions.git"
+    else:
+        raise NotImplementedError(clone_type)
+    _run_git("clone", clone_url)
+    if os.path.isdir("./AnalysisProductions"):
+        os.chdir("./AnalysisProductions")
+        _checkout(
+            version, working_group, production_name, branch_name, allow_deletes=True
+        )
+    else:
+        raise click.ClickException(
+            f"Cloning the repository failed! Check you have the correct permissions,"
+            f" to clone via {clone_type} or choose a different clone type [ssh, https, krb5]"
+        )
+
+
+@main.command()
+@click.argument("version", type=str, nargs=1)
+@click.argument("working_group", type=str, nargs=1)
+@click.argument("production_name", type=production_name_type, nargs=1)
+@click.argument("branch_name", type=str, nargs=1)
+@click.option("--allow-deletes/--no-allow-deletes", default=False)
+def checkout(
+    version: str,
+    working_group: str,
+    production_name: production_name_type,
+    branch_name: str,
+    allow_deletes: bool,
+) -> None:
+    """Clean out the current copy of the specified production and clone the production from the specified version \n
+    List the available tags for a specific production by running 'lb-ap versions <YOUR_PRODUCTION>'"""
+    _checkout(
+        version,
+        working_group,
+        production_name,
+        branch_name,
+        allow_deletes=allow_deletes,
+    )
+
+
+def _checkout(
+    version: str,
+    working_group: str,
+    production_name: production_name_type,
+    branch_name: str,
+    *,
+    allow_deletes: bool,
+):
+    inside_ap_datapkg()
+    repo_dir = Path.cwd()
+    if version not in production_to_versions(working_group, production_name):
+        raise click.ClickException(
+            f"Version {version} does not correspond to the {working_group} {production_name} production!"
+        )
+
+    proc = _run_git("status", "--porcelain")
+    if proc.stdout.strip():
+        raise click.ClickException(
+            f"Current git repository is not clean: {proc.stdout}"
+        )
+    _run_git("fetch", "origin")
+    _run_git("checkout", "-b", branch_name, "origin/master")
+
+    for child in repo_dir.iterdir():
+        if child.name.lower() != production_name.lower():
+            continue
+        if allow_deletes:
+            shutil.rmtree(child)
+        else:
+            raise FileExistsError(
+                f"{child.name} already exists in {repo_dir}. If you're ABSOLUTELY "
+                "sure you want to delete it, run this command with --allow-deletes."
+            )
+
+    proc = _run_git("ls-tree", "-d", version, printing=False)
+    for line in proc.stdout.strip().splitlines():
+        _, git_prod_name = line.decode().split("\t")
+        if git_prod_name.lower() == production_name.lower():
+            break
+    else:
+        raise click.ClickException(
+            f"Failed to find match for {production_name} in {version}"
+        )
+    click.secho(f"Found match for {production_name} -> {git_prod_name} in {version}!")
+
+    click.secho(f"Checking out the {version} version of {production_name}", fg="yellow")
+    _run_git("checkout", version, "--", git_prod_name)
+
+
+def _run_git(
+    *args: Iterable[str], printing: bool = True
+) -> subprocess.CompletedProcess:
+    """Run a git command and return the output."""
+    cmd = ("git",) + args
+    click.secho(f"Running {shlex.join(cmd)}", bold=True)
+    proc = logging_subprocess_run(cmd, printing=printing)
+    if proc.returncode != 0:
+        raise click.ClickException(f"Failed to run git command: {proc.args}")
+    return proc
+
+
 @main.command()
 @click.argument("production_name", type=production_name_type, default="", nargs=1)
 def list(production_name):
-    """List the available production folders by running lb-ap list \n
-    List the available productions for a specific production by running lb-ap list <YOUR_PRODUCTION>"""
+    """List the available production folders by running 'lb-ap list' \n
+    List the available productions for a specific production by running 'lb-ap list <YOUR_PRODUCTION>'"""
     inside_ap_datapkg()
 
     if production_name:
