@@ -21,11 +21,13 @@ from os.path import join
 
 import click
 import requests
-from LbAPCommon import get_sso_token, parse_yaml, render_yaml, validate_yaml
+from LbAPCommon import get_api_token, parse_yaml, render_yaml, validate_yaml
 from LbAPCommon.hacks import project_uses_cmt
 from LbAPCommon.options_parsing import validate_options
 
 from .utils import (
+    LBAPI_CLIENT_ID,
+    LBAPI_HOST,
     check_production,
     check_status_code,
     create_output_dir,
@@ -34,13 +36,10 @@ from .utils import (
     resolve_input_overrides,
 )
 
-LBAPI_HOST = os.environ.get("LBAPI_HOST", "https://lbap.app.cern.ch")
-LBAPI_CLIENT_ID = os.environ.get("LBAPI_CLIENT_ID", "lhcb-analysis-productions")
-
 
 def prepare_test(production_name, job_name, dependent_input=None):
     """Run a local test job for a specific production job"""
-    token = get_sso_token(LBAPI_HOST, LBAPI_CLIENT_ID)
+    token = get_api_token(LBAPI_CLIENT_ID, f"{LBAPI_HOST}/user")
 
     # Check if production exists
     check_production(production_name)
@@ -72,6 +71,8 @@ def prepare_test(production_name, job_name, dependent_input=None):
     }
     if isinstance(job_data["options"], dict):
         prodrun_conf["options"] = job_data["options"].copy()
+        if "entrypoint" not in job_data["options"]:
+            prodrun_conf["options"].setdefault("format", "WGProd")
     else:
         prodrun_conf["options"] = {"format": "WGProd", "files": job_data["options"]}
 
@@ -144,7 +145,7 @@ def prepare_test(production_name, job_name, dependent_input=None):
 
 
 def prepare_reproduce(pipeline_id, production_name, job_name, test_id):
-    token = get_sso_token(LBAPI_HOST, LBAPI_CLIENT_ID)
+    token = get_api_token(LBAPI_CLIENT_ID, f"{LBAPI_HOST}/user")
 
     click.secho(
         f"Reproducing test for test {pipeline_id} {production_name} {job_name}",
