@@ -8,16 +8,17 @@
 # granted to it by virtue of its status as an Intergovernmental Organization  #
 # or submit itself to any jurisdiction.                                       #
 ###############################################################################
+import asyncio
 import glob
 import os
 import pathlib
 import re
 import subprocess
 import tempfile
-from concurrent.futures import ThreadPoolExecutor
 from datetime import datetime
 from os.path import dirname, join
 
+import apd
 import click
 from LbAPCommon import write_jsroot_compression_options
 from LbAPCommon.hacks import setup_lbrun_environment
@@ -102,6 +103,15 @@ def check_production(production_name):
         )
 
 
+def check_production_existed(available_versions: dict, production_name: str):
+    for productions in list(available_versions.values()):
+        if production_name in productions:
+            return True
+    raise click.ClickException(
+        f"Can't find production {production_name} for any version tag of AnalysisProductions."
+    )
+
+
 def create_output_dir(production_name, require_cmt):
     """Create the directory structure for testing locally"""
     date_string = datetime.now().strftime("%Y-%m-%d-%H-%M-%S")
@@ -134,28 +144,39 @@ def create_output_dir(production_name, require_cmt):
     return dynamic_dir, out_dir
 
 
-def log_popen_pipe(p, pipe_name):
+def logging_subprocess_run(args, *, shell=False, cwd=None, printing=True):
+    return asyncio.get_event_loop().run_until_complete(
+        _logging_subprocess_run(*args, shell=shell, cwd=cwd, printing=printing)
+    )
+
+
+async def _logging_subprocess_run(*command, shell, cwd, printing):
+    proc = await asyncio.create_subprocess_exec(
+        *command,
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        shell=shell,
+        cwd=cwd,
+    )
+    stdout, stderr, returncode = await asyncio.gather(
+        _handle_output(proc.stdout, printing),
+        _handle_output(proc.stderr, printing),
+        proc.wait(),
+    )
+    return subprocess.CompletedProcess(command, returncode, stdout, stderr)
+
+
+async def _handle_output(stream, printing: bool):
+    """Process the output of a current local execution"""
     result = b""
-    while p.poll() is None:
-        line = getattr(p, pipe_name).readline()
+    while line := await stream.readline():
         result += line
-        click.echo(line.decode(errors="backslashreplace").rstrip("\n"))
+        if printing:
+            click.echo(line.decode(errors="backslashreplace"), nl=False)
     return result
 
 
-def logging_subprocess_run(args, *, shell=False, cwd=None):
-    with subprocess.Popen(
-        args, shell=shell, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd
-    ) as p:
-        with ThreadPoolExecutor(2) as pool:
-            stdout = pool.submit(log_popen_pipe, p, "stdout")
-            stderr = pool.submit(log_popen_pipe, p, "stderr")
-            stdout = stdout.result()
-            stderr = stderr.result()
-    return subprocess.CompletedProcess(args, p.returncode, stdout, stderr)
-
-
-def production_name_type(production_name):
+def production_name_type(production_name: str) -> str:
     return production_name.strip("/")
 
 
@@ -240,3 +261,15 @@ def check_status_code(response):
         click.secho(response.text, fg="red")
         raise click.ClickException("Network request failed")
     return response
+
+
+# def version_to_production(version: str) -> list:  # TODO make version_to_production
+#    """Find the production name associated with a particular AnalysisProductions tag."""
+
+
+def production_to_versions(working_group: str, production_name: str) -> list:
+    """List the AnalysisProductions tags corresponding to the specified production."""
+    datasets = apd.AnalysisData(working_group, production_name)
+    versions = list(datasets.available_tags["version"])
+    versions.sort()
+    return versions
